
import json
from copy import deepcopy as copy

from .div import Div
from .title import Title
from .tab import Tab
from .list_js import ListJs
from .misc import Misc
from .header import Header


class Dashboard:

    def __init__(self, verbose=False, **kwargs):
        """
        TBD
        """
        # attributes
        self.type = 'Dashboard'
        self.title = None
        self.tabs = []
        self.css = None
        self.js = None
        self.misc = None
        self.header = None
        self.markdown = None
        self.widgets = False
        self.widgets_state = None
        self.latex = None

        for k, v in kwargs.items():
            setattr(self, k, v)
        self.valid = self.check(verbose=verbose)

        self.build_widgets_state()

    def check(self, verbose=False):
        isTitle = (self.title is None) or isinstance(self.title, Title)
        isTabs = isinstance(self.tabs, list) and all(
            isinstance(e, Tab) for e in self.tabs)
        isJs = isinstance(self.js, ListJs)
        isMisc = isinstance(self.misc, Misc)
        isExactlyOneActiveTab = sum([1 for e in self.tabs if e.active]) == 1
        isHeader = isinstance(self.header, Header) or self.header is None
        isMarkdown = isinstance(self.markdown, bool)
        isWidgets = isinstance(self.widgets, bool)
        isWidgetsState = isinstance(
            self.widgets_state, list) and all(
            isinstance(e, dict) for e in self.widgets_state)
        isLatex = isinstance(self.latex, bool)
        isDashboard = isTitle and isTabs and isMisc and isExactlyOneActiveTab \
            and isHeader and isMarkdown and isWidgets and isLatex
        if verbose:
            print('Dashboard: isTitle=', isTitle)
            print('Dashboard: isTabs=', isTabs)
            print('Dashboard: isJs=', isJs)
            print('Dashboard: isMisc=', isMisc)
            print('Dashboard: isExactlyOneActiveTab=', isExactlyOneActiveTab)
            print('Dashboard: isHeader=', isHeader)
            print('Dashboard: isMarkdown=', isMarkdown)
            print('Dashboard: isWidgets=', isWidgets)
            print('Dashboard: isWidgetsState=', isWidgetsState)
            print('Dashboard: isLatex=', isLatex)
            print('Dashboard: isDashboard=', isDashboard)
        msg = 'Dashboard attributes title, tabs, css, js, misc must be objects ' + \
              'resp. Title, list(Tab), str, ListJs, Misc, Header, list(Div), bool. ' + \
              'There must be exactly one active tab'
        assert isDashboard, msg

        return True

    def build_widgets_state(self):
        """
        """
        if self.widgets_state and len(self.widgets_state):
            d0 = self.widgets_state[0]
            for d in self.widgets_state[1:]:
                d0['state'].update(d['state'])
            self.widgets_state = d0
            self.widgets_state = json.dumps(self.widgets_state)
        else:
            self.widgets_state = ''

    def to_dict(self, short=False):
        d = copy(self.__dict__)
        d = {k: v for k, v in d.items() if v is not None}
        for k, v in d.items():
            if isinstance(v, (Title, Misc)):
                d[k] = v.to_dict()
            elif isinstance(v, Header):
                d[k] = v.to_dict(short=short)
            elif isinstance(v, ListJs):
                d[k] = v.to_list()
            elif isinstance(v, (str, bool)):
                d[k] = v
            elif isinstance(v, list):
                v2 = []
                for e in v:
                    if isinstance(e, (Tab, Div)):
                        v2.append(e.to_dict())
                d[k] = v2
        return d

    def pprint(self, indent=2):
        d = self.to_dict(short=True)
        d = json.dumps(d, sort_keys=True, indent=indent)
        print(d)

    def __repr__(self):
        return str(self.to_dict())




import sys
import os
import re
import jinja2 as jj

dir_file = os.path.dirname(os.path.abspath(__file__))
dir_template = os.path.join(dir_file, 'templates')

loader = jj.FileSystemLoader(dir_template)

env = jj.Environment(loader=loader,
                     variable_start_string='__$',
                     variable_end_string='$__',
                     block_start_string='{%',
                     block_end_string='%}'
                     )


def build(data=None, save=True, save_path='saved', save_name='index.html', server_port=8081):
    """
    TBD
    """
    template_name = 'dashboard.template.html'
    template = env.get_template(template_name)
    content = template.render(data=data)

    if not save:
        return content

    if not os.path.exists(save_path):
        os.makedirs(save_path)

    # save dasboard
    file_path = os.path.join(save_path, save_name)
    with open(file_path, 'w', encoding='utf-8') as f:
        f.write(content)
    print('file {} created on disk'.format(file_path))

    # determine OS
    if sys.platform in ['darwin', 'linux']:
        suffix = 'sh'
    elif sys.platform == 'win32':
        suffix = 'bat'
    else:
        print('OS unknown: No web server launch script created')

    # launch web server script content
    content = 'python -m http.server {}'.format(server_port)

    # save script
    file_path = os.path.join(save_path, 'server.' + suffix)
    with open(file_path, 'w') as f:
        f.write(content)
    print('file {} created on disk:'.format(file_path))
    print('\t{}'.format(content))
    print('Run it to launch web server and test dashboard')


def minify(content):
    # remove js or css /* blah */ comments
    output = re.sub('/\*(.|\n)*?\*/', ' ', content)

    # remove js // blah comments
    output = re.sub('//\s.*?\n', ' ', output)

    # substitute all tabs, newlines and other
    # whitespace-like characters with single space
    # .strip() in the end will cut off any trailing whitespace
    output = re.sub('\s+', ' ', output).strip()

    return output




import json
from copy import deepcopy as copy


class Title:

    def __init__(self, verbose=False, **kwargs):
        """
        TBD
        """
        # attributes
        self.type = 'Title'
        self.id_name = None
        self.class_name = None
        self.size = None
        self.text = ''

        for k, v in kwargs.items():
            setattr(self, k, v)
        self.valid = self.check(verbose=verbose)

    def check(self, verbose=False):
        isSize = isinstance(self.size, int) and self.size in [1, 2, 3, 4, 5]
        isText = isinstance(self.text, str) and self.text != ''
        isTitle = isSize and isText

        if verbose:
            print('Title: isTitle=', isTitle)
        msg = 'Title must contain size and text'
        assert isTitle, msg
        return True

    def to_dict(self):
        d = copy(self.__dict__)
        d = {k: v for k, v in d.items() if v is not None}
        return d

    def pprint(self, indent=2):
        d = json.dumps(self.to_dict(), sort_keys=True, indent=indent)
        print(d)

    def __repr__(self):
        return str(self.to_dict())




import json
from copy import deepcopy as copy

from .list_div import ListDiv


class Row:

    def __init__(self, verbose=False, **kwargs):
        """
        TBD
        """
        # attributes
        self.type = 'Row'
        self.id_name = None
        self.class_name = None
        self.elmts = None

        for k, v in kwargs.items():
            setattr(self, k, v)
        self.valid = self.check(verbose=verbose)

    def check(self, verbose=False):
        isRow = isinstance(self.elmts, ListDiv)
        if verbose:
            print('Rows: isRow=', isRow)
        msg = 'Row must contain a ListDiv instance'
        assert isRow, msg

        return True

    def to_dict(self):
        d = copy(self.__dict__)
        d = {k: v for k, v in d.items() if v is not None}
        for k, v in d.items():
            if isinstance(v, ListDiv):
                d[k] = v.to_dict()
        return d

    def pprint(self, indent=2):
        d = json.dumps(self.to_dict(), sort_keys=True, indent=indent)
        print(d)

    def __repr__(self):
        return str(self.to_dict())




import json
from copy import deepcopy as copy

from .title import Title
from .row import Row


class Tab:

    def __init__(self, verbose=False, **kwargs):
        """
        TBD
        """
        # attributes
        self.type = 'Tab'
        self.class_name = None
        self.name = None
        self.active = False
        ## rows or tabs
        self.elmts = None
        self.level = 1
        self.keyboard = True

        for k, v in kwargs.items():
            setattr(self, k, v)
        self.valid = self.check(verbose=verbose)

        self.keyboard = int(self.keyboard)

    def check(self, verbose=False):
        isLevel = isinstance(self.level, int) and self.level in [1, 2]
        isCandListTab = isinstance(self.elmts, list) and all(
            isinstance(e, Tab) for e in self.elmts)
        isCandListTitleRow = isinstance(self.elmts, list) and all(
            (isinstance(e, Title) or isinstance(e, Row))
            for e in self.elmts)
        nbActiveTab = sum(
            [1 for e in self.elmts if (isinstance(e, Tab) and e.active)])
        isNbActiveTabCorrect = nbActiveTab == 1 if self.elmts[0].type == 'Tab' else True
        isKeyboard = isinstance(self.keyboard, bool)
        isTab = ((isLevel == 2 and isCandListTitleRow) or (
            isLevel == 1 and (isCandListTab or isCandListTitleRow))) and isKeyboard
        isTab = isTab and isNbActiveTabCorrect
        if verbose:
            print('Tab: isLevel=', isLevel)
            print('Tab: isCandListTab=', isCandListTab)
            print('Tab: isCandListTitleRow=', isCandListTitleRow)
            print('Tab: nbActiveTab=', nbActiveTab)
            print('Tab: isKeyboard=', isKeyboard)
            print('Tab: isNbActiveTabCorrect=', isNbActiveTabCorrect)
            print('Tab: isTab=', isTab)
        msg = 'Tab must contain a list of only (Title or Row) or only (Tab) instances (if level=1)\n' + \
              'Tab must contain a list of only (Title or Row) instances (if level=2)\n' +\
              'Only one underlying tab, if any, must be active=True'

        assert isTab, msg

        return True

    def to_dict(self):
        d = copy(self.__dict__)
        d = {k: v for k, v in d.items() if v is not None}
        for k, v in d.items():
            if isinstance(v, list):
                v2 = []
                for e in v:
                    if isinstance(e, (Title, Row, Tab)):
                        v2.append(e.to_dict())
                d[k] = v2

        return d

    def pprint(self, indent=2):
        d = json.dumps(self.to_dict(), sort_keys=True, indent=indent)
        print(d)

    def __repr__(self):
        return str(self.to_dict())




import os
import json
import base64

from copy import deepcopy as copy


class Header:

    def __init__(self, verbose=False, **kwargs):
        """
        TBD
        """
        # attributes
        self.type = 'Header'
        self.left_logo = None
        self.left_title = None
        self.right_logo = None
        self.toggle = False

        self.left_logo_img_src = None
        self.right_logo_img_src = None

        for k, v in kwargs.items():
            setattr(self, k, v)

        self.valid = self.check(verbose=verbose)
        self.build_img()

    def check(self, verbose=False):
        left_ok = (isinstance(self.left_logo, str) and
                   os.path.exists(self.left_logo)) or self.left_logo is None
        right_ok = (isinstance(self.right_logo, str)
                    and os.path.exists(self.right_logo)) or self.right_logo is None
        isTitle = isinstance(self.left_title, str) or self.left_title is None
        isToggle = isinstance(self.toggle, bool)

        isHeader = all([left_ok, right_ok, isTitle, isToggle])
        if verbose:
            print('Header: left_ok=', left_ok)
            print('Header: right_ok=', right_ok)
            print('Header: isTitle=', isTitle)
            print('Header: isToggle=', isToggle)
        msg = 'Header attributes if defined must be strings - preferably representing img paths and text'
        assert isHeader, msg

        return True

    def build_img(self):
        if self.left_logo:
            data_uri = base64.b64encode(open(self.left_logo, 'rb').read()).decode(
                'utf-8').replace('\n', '')
            self.left_logo_img_src = 'data:image/png;base64,{0}'.format(
                data_uri)

        if self.right_logo:
            data_uri = base64.b64encode(open(self.right_logo, 'rb').read()).decode(
                'utf-8').replace('\n', '')
            self.right_logo_img_src = 'data:image/png;base64,{0}'.format(
                data_uri)

    def to_dict(self, short=False, n=250):
        d = copy(self.__dict__)
        if short:
            for k in ['left_logo_img_src', 'right_logo_img_src']:
                e = d[k]
                d[k] = e[:n] + '...' + \
                    '({} more characters)'.format(len(e) - n)
        return d

    def pprint(self, indent=2):
        d = self.to_dict(short=True)
        d = json.dumps(d, sort_keys=True, indent=indent)
        print(d)

    def __repr__(self):
        d = str(self.to_dict())
        return d




import json
from copy import deepcopy as copy


class Misc:

    def __init__(self, verbose=False, **kwargs):
        """
        TBD
        """
        # attributes
        self.type = 'Misc'
        self.main_type = 'container-fluid'
        self.main_nav_width = None
        self.main_nav_min_height = None
        self.main_content_width = None
        self.no_margins = True

        for k, v in kwargs.items():
            setattr(self, k, v)
        self.valid = self.check(verbose=verbose)

    def check(self, verbose=False):
        t1 = self.main_type in ['container-fluid', 'container']
        t2 = isinstance(self.main_nav_width,
                        str) or self.main_nav_width is None
        t3 = isinstance(self.main_nav_min_height,
                        str) or self.main_nav_min_height is None
        t4 = isinstance(self.main_content_width,
                        str) or self.main_content_width is None
        t5 = isinstance(self.no_margins,
                        bool) or self.no_margins is None
        isMisc = all([t1, t2, t3, t4, t5])
        if verbose:
            print('Misc: t1=', t1)
            print('Misc: t2=', t2)
            print('Misc: t3=', t3)
            print('Misc: t4=', t4)
            print('Misc: t5=', t5)
            print('Misc: isMisc=', isMisc)
        msg = 'Mics attributes if defined must be strings - preferably representing valid css properties'
        assert isMisc, msg

        return True

    def to_dict(self):
        d = copy(self.__dict__)
        d = {k: v for k, v in d.items() if v is not None}
        return d

    def pprint(self, indent=2):
        d = json.dumps(self.to_dict(), sort_keys=True, indent=indent)
        print(d)

    def __repr__(self):
        d = str(self.to_dict())
        return d




import json
from copy import deepcopy as copy


class ListJs:

    def __init__(self, li_js_lib, verbose=False):
        """
        TBD
        """
        # attributes
        self.type = 'ListJs'
        self.li_js_lib = li_js_lib

        self.valid = self.check(verbose=verbose)

    def check(self, verbose=False):

        isList = isinstance(self.li_js_lib, list)
        isJsStr = all((isinstance(e, str)) for e in self.li_js_lib)
        isListJs = isList and isJsStr
        if verbose:
            print('ListJs: isList=', isList)
            print('ListJs: isJsStr=', isJsStr)
            print('ListJs: isListJs=', isListJs)
            
        msg = 'ListJs must be a list of strings, possibly empty' + \
              'each representing the url of a js lib'
        assert isListJs, msg

        return True

    def pprint(self, indent=2):
        d = json.dumps(self.to_list(), sort_keys=True, indent=indent)
        print(d)

    def to_list(self):
        d = copy(self.li_js_lib)
        return d

    def __repr__(self):
        return str(self.to_list())




import json
import markdown

from copy import deepcopy as copy
from mdx_gfm import GithubFlavoredMarkdownExtension


class Div:
    """"""

    def __init__(self, verbose=False, **kwargs):
        """
        TBD
        """
        # attributes
        self.type = 'Div'
        self.id_name = None
        self.class_name = None
        self.content = None
        self.content_md = None
        self.width = None
        self.is_markdown = False

        for k, v in kwargs.items():
            setattr(self, k, v)
        self.valid = self.check(verbose=verbose)

        # print('T1')
        # print(self.content_md)
        if self.is_markdown:
            md = markdown.Markdown(extensions=[GithubFlavoredMarkdownExtension()])
            # print('********* inside')
            self.content = md.convert(self.content)
            if verbose:
                print('Div {} markdown content converted to html'.format(self.id_name))
        # print('T2')
        # print(self.content_md)
        # print('end T2')

    def check(self, verbose=False):
        isDivProp = any([getattr(self, k) is not None for k in [
            'id_name', 'class_name', 'content', 'width', 'is_markdown']])
        isIdMarkdownDefined = (self.is_markdown and self.id_name) or True
        isDiv = isDivProp and isIdMarkdownDefined
        if verbose:
            print('Div: isDivProp=', isDivProp)
            print('Div: isIdMarkdownDefined=', isIdMarkdownDefined)
            print('Div: isDiv=', isDiv)
        msg = 'Div must contain a div element defined by at least one attribute\n' + \
              'Div id_name must be defined if is_markdown is True'

        assert isDiv, msg

        return True

    def to_dict(self):
        d = copy(self.__dict__)
        d = {k: v for k, v in d.items() if v is not None}
        return d

    def pprint(self, indent=2):
        d = json.dumps(self.to_dict(), sort_keys=True, indent=indent)
        print(d)

    def __repr__(self):
        return str(self.to_dict())




from .div import Div
from .list_div import ListDiv
from .title import Title
from .row import Row
from .tab import Tab
from .misc import Misc
from .list_js import ListJs
from .header import Header
from .dashboard import Dashboard




__name__ = 'ezdashboard'
name_url = __name__.replace('_', '-')

__version__ = '0.3.6'

__description__ = 'Easy dashboard creaation from raw HTML components'
__author__ = 'oscar6echo'
__author_email__ = 'olivier.borderies@sgcib.com'
__url__ = 'https://gitlab.com/oscar6echo/{}'.format(name_url)
__download_url__ = 'https://gitlab.com/oscar6echo/{}/repository/archive.tar.gz?ref={}'.format(name_url,
                                                                                              __version__)
__keywords__ = ['python', 'dashboard', 'jinja', 'html', 'js', 'css']
__license__ = 'MIT'
__classifiers__ = ['Development Status :: 4 - Beta',
                   'License :: OSI Approved :: MIT License',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6'
                  ]
__include_package_data__ = True
__package_data__ = {
    'templates': [
        'templates/*.html',
        'templates/*.css'
    ]
}
__zip_safe__ = False
__entry_points__ = {}



import json
from copy import deepcopy as copy

from .div import Div

class ListDiv:

    def __init__(self, verbose=False, **kwargs):
        """
        TBD
        """
        # attributes
        self.type = 'ListDiv'
        self.id_name = None
        self.class_name = None
        self.width = 12
        self.elmts = []
        self.level = 1

        for k, v in kwargs.items():
            setattr(self, k, v)
        self.valid = self.check(verbose=verbose)

    def check(self, verbose=False):
        isListDivLevel1 = isinstance(self.elmts, list) and all(
            (isinstance(e, (Div, ListDiv))) for e in self.elmts)
        isListDivLevel2 = isinstance(self.elmts, list) and all(
            (isinstance(e, Div)) for e in self.elmts)
        isListDiv = (self.level == 1 and isListDivLevel1) or (
            self.level == 2 and isListDivLevel2)
        if verbose:
            print('ListDiv: isListDivLevel1=', isListDivLevel1)
            print('ListDiv: isListDivLevel2=', isListDivLevel2)
            print('ListDiv: isListDiv=', isListDiv)
        msg = 'ListDiv must contain a list of Div or ListDiv instances only'
        assert isListDiv, msg

        return True

    def to_dict(self):
        d = copy(self.__dict__)
        d = {k: v for k, v in d.items() if v is not None}
        for k, v in d.items():
            if isinstance(v, list):
                v2 = []
                for e in v:
                    if isinstance(e, (Div, ListDiv)):
                        v2.append(e.to_dict())
                d[k] = v2
        return d

    def pprint(self, indent=2):
        d = json.dumps(self.to_dict(), sort_keys=True, indent=indent)
        print(d)

    def __repr__(self):
        return str(self.to_dict())
